@@ -157,17 +157,22 @@ def get_colors():
 # Define the 'overview plots', i.e. the most useful set of plots to explore different aspects of a simulation
 overview_plots = [
             'cum_infections',
-            'cum_diagnoses',
             'cum_severe',
             'cum_critical',
             'cum_deaths',
+            'cum_diagnoses',
             'new_infections',
-            'new_tests',
-            'new_diagnoses',
+            'new_severe',
+            'new_critical',
             'new_deaths',
-            'new_quarantined',
+            'new_diagnoses',
             'n_infectious',
+            'n_severe',
+            'n_critical',
+            'n_susceptible',
+            'new_tests',
             'n_symptomatic',
+            'new_quarantined',
             'n_quarantined',
             'test_yield',
             'r_eff',
