@@ -99,12 +99,31 @@ def compute_infections(beta,     sources,  targets,   layer_betas, rel_trans,  r
     return source_inds, target_inds
 
 
+@nb.njit((nbint[:], nbint[:], nb.int64[:]), cache=True)
+def find_contacts(p1, p2, inds):
+    """
+    Numba for Layer.find_contacts()
+
+    A set is returned here rather than a sorted array so that custom tracing interventions can efficiently
+    add extra people. For a version with sorting by default, see Layer.find_contacts(). Indices must be
+    an int64 array since this is what's returned by true() etc. functions by default.
+    """
+    pairing_partners = set()
+    inds = set(inds)
+    for i in range(len(p1)):
+        if p1[i] in inds:
+            pairing_partners.add(p2[i])
+        if p2[i] in inds:
+            pairing_partners.add(p1[i])
+    return pairing_partners
+
+
 #%% Sampling and seed methods
 
 __all__ += ['sample', 'get_pdf', 'set_seed']
 
 
-def sample(dist=None, par1=None, par2=None, size=None):
+def sample(dist=None, par1=None, par2=None, size=None, **kwargs):
     '''
     Draw a sample from the distribution specified by the input.
 
@@ -113,6 +132,7 @@ def sample(dist=None, par1=None, par2=None, size=None):
         par1 (float): the "main" distribution parameter (e.g. mean)
         par2 (float): the "secondary" distribution parameter (e.g. std)
         size (int):   the number of samples (default=1)
+        kwargs (dict): passed to individual sampling functions
 
     Returns:
         A length N array of samples
@@ -141,17 +161,17 @@ def sample(dist=None, par1=None, par2=None, size=None):
 
     # Compute distribution parameters and draw samples
     # NB, if adding a new distribution, also add to choices above
-    if   dist == 'uniform':       samples = np.random.uniform(low=par1, high=par2, size=size)
-    elif dist == 'normal':        samples = np.random.normal(loc=par1, scale=par2, size=size)
-    elif dist == 'normal_pos':    samples = np.abs(np.random.normal(loc=par1, scale=par2, size=size))
-    elif dist == 'normal_int':    samples = np.round(np.abs(np.random.normal(loc=par1, scale=par2, size=size)))
-    elif dist == 'poisson':       samples = n_poisson(rate=par1, n=size) # Use Numba version below for speed
-    elif dist == 'neg_binomial':  samples = n_neg_binomial(rate=par1, dispersion=par2, n=size) # Use custom version below
+    if   dist == 'uniform':       samples = np.random.uniform(low=par1, high=par2, size=size, **kwargs)
+    elif dist == 'normal':        samples = np.random.normal(loc=par1, scale=par2, size=size, **kwargs)
+    elif dist == 'normal_pos':    samples = np.abs(np.random.normal(loc=par1, scale=par2, size=size, **kwargs))
+    elif dist == 'normal_int':    samples = np.round(np.abs(np.random.normal(loc=par1, scale=par2, size=size, **kwargs)))
+    elif dist == 'poisson':       samples = n_poisson(rate=par1, n=size, **kwargs) # Use Numba version below for speed
+    elif dist == 'neg_binomial':  samples = n_neg_binomial(rate=par1, dispersion=par2, n=size, **kwargs) # Use custom version below
     elif dist in ['lognormal', 'lognormal_int']:
         if par1>0:
             mean  = np.log(par1**2 / np.sqrt(par2 + par1**2)) # Computes the mean of the underlying normal distribution
             sigma = np.sqrt(np.log(par2/par1**2 + 1)) # Computes sigma for the underlying normal distribution
-            samples = np.random.lognormal(mean=mean, sigma=sigma, size=size)
+            samples = np.random.lognormal(mean=mean, sigma=sigma, size=size, **kwargs)
         else:
             samples = np.zeros(size)
         if dist == 'lognormal_int':
@@ -332,7 +352,7 @@ def n_poisson(rate, n):
     return np.random.poisson(rate, n)
 
 
-def n_neg_binomial(rate, dispersion, n): # Numba not used due to incompatible implementation
+def n_neg_binomial(rate, dispersion, n, step=1): # Numba not used due to incompatible implementation
     '''
     An array of negative binomial trials; with dispersion = ∞, converges to Poisson.
 
@@ -340,14 +360,16 @@ def n_neg_binomial(rate, dispersion, n): # Numba not used due to incompatible im
         rate (float): the rate of the process (mean, same as Poisson)
         dispersion (float): amount of dispersion: 0 = infinite, 1 = std is equal to mean, ∞ = Poisson
         n (int): number of trials
+        step (float): the step size to use if non-integer outputs are desired
 
     **Example**::
 
         outcomes = cv.n_neg_binomial(100, 1, 20) # 20 negative binomial trials with mean 100 and dispersion equal to mean
     '''
     nbn_n = dispersion
-    nbn_p = dispersion/(rate + dispersion)
-    return np.random.negative_binomial(n=nbn_n, p=nbn_p, size=n)
+    nbn_p = dispersion/(rate/step + dispersion)
+    samples = np.random.negative_binomial(n=nbn_n, p=nbn_p, size=n)*step
+    return samples
 
 
 @nb.njit((nbint, nbint), cache=True) # This hugely increases performance
@@ -408,7 +430,7 @@ def choose_w(probs, n, unique=True):
 
 #%% Simple array operations
 
-__all__ += ['true',   'false',   'defined',
+__all__ += ['true',   'false',   'defined', 'undefined',
             'itrue',  'ifalse',  'idefined',
             'itruei', 'ifalsei', 'idefinedi']
 
@@ -456,6 +478,20 @@ def defined(arr):
     return (~np.isnan(arr)).nonzero()[0]
 
 
+def undefined(arr):
+    '''
+    Returns the indices of the values of the array that are not-nan.
+
+    Args:
+        arr (array): any array
+
+    **Example**::
+
+        inds = cv.defined(np.array([1,np.nan,0,np.nan,1,0,1]))
+    '''
+    return np.isnan(arr).nonzero()[0]
+
+
 def itrue(arr, inds):
     '''
     Returns the indices that are true in the array -- name is short for indices[true]
